;***************************************************************************
;*   Library Name: Sounds
;*       Coded By: Derlidio Siqueira & John Logsdon
;*   Last Updated: 8/11/2012
;*        Purpose: Handles the sounds
;***************************************************************************


;****************************************************
 ;  Procedure: MyPlaySound()
 ;     Author: Derlidio Siqueira & John Logsdon
 ;   Last Upd: 2/07/2005
 ;    Purpose: Adds a sound to the Sounds list and
 ;                    randomizes it's volume, pan, and frequency values
 ;       Args: Sound Type
 ;    Returns: n/a
 ;   Comments: None
 ;****************************************************
 Procedure MyPlaySound(SoundType.l)
    Volume = 100
    Frequency = 22000
    Select SoundType 
       Case 1
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?LaserSound)
          ; then set the approximate time for the sound
          Length.l = 300
          Frequency = 30000
        Case 2
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?LaserSound)
          ; then set the approximate time for the sound
          Length.l = 300
          Frequency = 12000
       Case 3
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?PlayerHitSound)
          ; then set the approximate time for the sound
          Length.l = 400
          Volume = 55 
        Case 4
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?PlayerDeadSound)
          ; then set the approximate time for the sound
          Length.l = 1000
       Case 5
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?RobotHitSound)
          ; then set the approximate time for the sound
          Length.l = 250
          Volume = 75 
       Case 6
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?RobotDeadSound)
          ; then set the approximate time for the sound
          Length.l = 1600
          Volume = 60 
        Case 7
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?PlayerSnoringSound)
          ; then set the approximate time for the sound
          Length.l = 1900
          Volume = 90
          Frequency = 35000
        Case 8
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?StartLevelSound)
          ; then set the approximate time for the sound
          Length.l = 3000
       Case 9
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?ExitLevelSound)
          ; then set the approximate time for the sound
          Length.l = 2400
          Volume = 75 
       Case 10
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?IntruderAlertSound)
          ; then set the approximate time for the sound
          Length.l = 1000
        Case 11
          ; first grab the sound out of memory
          ID = CatchSound(#PB_Any,?HealthSound)
          ; then set the approximate time for the sound
          Length.l = 3000
          Volume = 60 
      EndSelect
   
    ; if a valid ID is found
    If ID
      ; add the sound to the list
      AddElement(Sounds())
      
      ; then populate the list
      Sounds()\ID = ID
      Sounds()\Length = Length.l
      Sounds()\StartTime = ElapsedMilliseconds()
      SoundFrequency(ID,Frequency)
      SoundVolume(ID,Volume)

      ; play the sound
      PlaySound(ID)
      
    EndIf
    
  EndProcedure
  
  
  
  ;****************************************************
  ;  Procedure: MyRemoveSound()
  ;     Author: Derlidio Siqueira 
  ;    Purpose: Removes a sound from the Sounds list
  ;       Args: n/a
  ;    Returns: n/a
  ;   Comments: This will kill a sound regardless if it is still playing or 
  ;                       not. It kills  the sound based on the time value
  ;                       given in MyPlaySound()
  ;****************************************************
  Procedure MyRemoveSound()

    ; run through each of the sounds
    ForEach(Sounds())
      ; if a particular one has expired (time-wise)
      If ElapsedMilliseconds() - Sounds()\StartTime > Sounds()\Length
        ; stop the sound, remove the sound, and delete the element 
        ; from the list
        StopSound(Sounds()\ID)
        FreeSound(Sounds()\ID)
        DeleteElement(Sounds())
      EndIf
    Next
  EndProcedure
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 4
; Folding = -